﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using System.Drawing;

namespace GradientionSample
{
    public class GradientionButton : Control
    {
        private void DrawButton(Graphics g, bool pressed, bool focused)
        {
            // ボタンの背景を描画
            Pen backLinePen;
            if (!focused)
            {
                backLinePen = new Pen(Color.Black);
            }
            else
            {
                // フォーカスが当たっている場合は、枠線を太く描画する
                backLinePen = new Pen(Color.Black, 2.5f);
            }

            // DrawRectangleメソッドで線を描画するが
            // 線の太さを考慮して幅と高さから引く
            Rectangle lineRect = new Rectangle();
            lineRect.X = ClientRectangle.X + (int)backLinePen.Width;
            lineRect.Y = ClientRectangle.Y + (int)backLinePen.Width;
            lineRect.Width = ClientRectangle.Width - (int)backLinePen.Width * 2;
            lineRect.Height = ClientRectangle.Height - (int)backLinePen.Width * 2;

            g.FillRectangle(new SolidBrush(Color.DimGray), ClientRectangle);
            g.DrawRectangle(backLinePen, lineRect);

            // ボタンの表示領域
            Rectangle btnRect = ClientRectangle;
            btnRect.X += 2;
            btnRect.Y += 2;
            btnRect.Width -= 4;
            btnRect.Height -= 4;

            Color stColor, edColor;
            if (!pressed)
            {
                stColor = Color.Red;
                edColor = Color.Black;
            }
            else
            {
                // タップされている場合は、色を反転させて描画する
                stColor = Color.Black;
                edColor = Color.Red;
            }

            // 角丸グラデーションボタンの描画
            GraphicsExtension.DrawGradientRoundRectButton(
                g, btnRect, stColor, edColor, Color.Black, FillDirection.TopToBottom);

            // ボタンに表示するテキストの表示(テキストは中央に表示)
            StringFormat sf = new StringFormat();
            sf.LineAlignment = StringAlignment.Center;
            sf.Alignment = StringAlignment.Center;
            g.DrawString(Text, Font, new SolidBrush(Color.White), btnRect, sf);
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            Capture = true;
            Focus();

            // 再描画要求
            Invalidate();
        }

        protected override void OnMouseUp(MouseEventArgs e)
        {
            Capture = false;

            // 再描画要求
            Invalidate();
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                Capture = true;
                Focus();
                // 再描画要求
                Invalidate();
            }
        }

        protected override void OnKeyUp(KeyEventArgs e)
        {
            Capture = false;

            // 再描画要求
            Invalidate();
        }

        protected override void OnGotFocus(EventArgs e)
        {
            base.OnGotFocus(e);

            // 再描画要求
            Invalidate();
        }

        protected override void OnLostFocus(EventArgs e)
        {
            base.OnLostFocus(e);

            // 再描画要求
            Invalidate();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            bool pressed = Capture;
            bool focused = Focused;

            DrawButton(e.Graphics, pressed, focused);
        }

        protected override void OnPaintBackground(PaintEventArgs e)
        {
            //base.OnPaintBackground(e);
        }
    }
}
